import ktc
import onnx, onnxruntime
from platform import platform
import sys, os
import glob
import cv2
import numpy as np

loaded_m = onnx.load("/home/APANet/APANetV1_0820.onnx")
result_m = ktc.onnx_optimizer.torch_exported_onnx_flow(loaded_m)
optimized_m = ktc.onnx_optimizer.onnx2onnx_flow(result_m)
onnx.save(optimized_m, "/home/APANet/optimized.onnx")

km = ktc.ModelConfig(32771, "0001", "520", onnx_model = optimized_m)
eval_result = km.evaluate()
print("\n========== evaluate result ========\n")
print(eval_result)

def preprocess(image, mean, scale):
    image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    image = cv2.resize(image, (224, 224))
    img_data = image.astype(np.float32)
    img_data = scale * (img_data - mean)
    return img_data

mean = np.asarray([123.675, 116.28, 103.53], dtype = np.float32)
scale = np.asarray([0.01712, 0.0175, 0.0174], dtype = np.float32)
input_file = "/home/APANet/images/output.bmp"
img = cv2.imread(input_file)

input_data = preprocess(img, mean, scale)
inf_result = ktc.kneron_inference([input_data], onnx_file = "/home/APANet/optimized.onnx", input_names = ["input.1"])
print("output shapes: {}".format([o.shape for o in inf_result]))


input_mapping = {"input.1" : [input_data]}
bie_path = km.analysis(input_mapping, output_bie = None, threads = 4)
print("bie path: {}".format(bie_path))

fixed_results = ktc.kneron_inference([input_data], bie_file = bie_path, input_names = ["input.1"], platform = 520)
print("bie inference output: {}".format([o.shape for o in fixed_results]))

compile_result = ktc.compile([km])
print("start hw simulation")
hw_results = ktc.kneron_inference([input_data], nef_file = compile_result, platform = 520)
print("hw output: {}".format([o.shape for o in hw_results]))
